@@ -3,13 +3,12 @@ package JMX::Jmx4Perl::Nagios::CheckJmx4Perl;
 use strict;
 use warnings;
 use JMX::Jmx4Perl::Nagios::SingleCheck;
-use JMX::Jmx4Perl::Nagios::MessageHandler;
 use JMX::Jmx4Perl;
 use JMX::Jmx4Perl::Request;
 use JMX::Jmx4Perl::Response;
 use Data::Dumper;
 use Nagios::Plugin;
-use Nagios::Plugin::Functions qw(:codes %ERRORS %STATUS_TEXT);
+use Nagios::Plugin::Functions qw(:codes %STATUS_TEXT);
 use Time::HiRes qw(gettimeofday tv_interval);
 use Carp;
 use Text::ParseWords;
@@ -91,38 +90,22 @@ sub execute {
             push @requests,@{$check->get_requests($jmx,\@ARGV)};            
         }
         my $responses = $self->_send_requests($jmx,@requests);
-        #print Dumper($responses);
         my @extra_requests = ();
         my $nr_checks = scalar(@{$self->{checks}});
         if ($nr_checks == 1) {
-            eval {
-                my @r = $self->{checks}->[0]->extract_responses($responses,\@requests,{ target => $target_config });
-                push @extra_requests,@r if @r;
-            };
-            $self->nagios_die($@) if $@;
+            my @r = $self->{checks}->[0]->extract_responses($responses,\@requests,{ target => $target_config });
+            push @extra_requests,@r if @r;
         } else {
             my $i = 1;
             for my $check (@{$self->{checks}}) {
                 # A check can consume more than one response
-                my $prefix = $self->_multi_check_prefix($check,$i++,$nr_checks);
-                eval {
-                    my @r = $check->extract_responses($responses,\@requests,
-                                                        { 
-                                                         target => $target_config, 
-                                                         prefix => $prefix,
-                                                         error_stat => $error_stat
-                                                        });
-                    push @extra_requests,@r if @r;
-                };
-                if ($@) {
-                    my $txt = $@;
-                    $txt =~ s/^(.*?)\n.*$/$1/s;
-                    my $code = $np->opts->{'unknown-is-critical'} ? CRITICAL : UNKNOWN;
-                    $check->update_error_stats($error_stat,$code);
-                    $prefix =~ s/\%c/$STATUS_TEXT{$code}/g;
-                    my $msg_handler = $np->{msg_handler} || $np; 
-                    $msg_handler->add_message($code,$prefix . $txt);
-                }
+                my @r = $check->extract_responses($responses,\@requests,
+                                                    { 
+                                                     target => $target_config, 
+                                                     prefix => $self->_multi_check_prefix($check,$i++,$nr_checks),
+                                                     error_stat => $error_stat
+                                                    });
+                push @extra_requests,@r if @r;
             }
         }
         # Send extra requests, e.g. for switching on the history
@@ -152,13 +135,12 @@ output, which can be extracted from NagiosPlugin object.
 
 =cut 
 
-sub do_exit {    
+sub do_exit {
     my $self = shift;
     my $error_stat = shift;
     my $np = $self->{np};
 
-    my $msg_handler = $np->{msg_handler} || $np; 
-    my ($code,$message) = $msg_handler->check_messages(join => "\n", join_all => "\n");
+    my ($code,$message) = $np->check_messages(join => "\n", join_all => "\n");
     ($code,$message) = $self->_prepare_multicheck_message($np,$code,$message,$error_stat) if scalar(@{$self->{checks}}) > 1;
     
     $np->nagios_exit($code, $message);
@@ -172,63 +154,28 @@ sub _prepare_multicheck_message {
     my $error_stat = shift;
 
     my $summary;
-    my $labels = $self->{multi_check_labels} || {};
     my $nr_checks = scalar(@{$self->{checks}});
-    $code = $self->_check_for_UNKNOWN($error_stat,$code);
     if ($code eq OK) {
-        $summary = $self->_format_multicheck_ok_summary($labels->{summary_ok} ||
-                                                        "All %n checks OK",$nr_checks);
+        $summary = "All " . $nr_checks . " checks OK";            
     } else {
-        $summary = $self->_format_multicheck_failure_summary($labels->{summary_failure} ||
-                                                             "%e of %n checks failed [%d]",
-                                                             $nr_checks,
-                                                             $error_stat);
-    }
-    return ($code,$summary . "\n" . $message);
-}
-
-# UNKNOWN shadows everything else
-sub _check_for_UNKNOWN {
-    my $self = shift;
-    my $error_stat = shift;
-    my $code = shift;
-    return $error_stat->{UNKNOWN} && scalar(@$error_stat->{UNKNOWN}) ? UNKNOWN : $code;
-}
-
-sub _format_multicheck_ok_summary {
-    my $self = shift;
-    my $format = shift;
-    my $nr_checks = shift;
-    my $ret = $format;
-    $ret =~ s/\%n/$nr_checks/g;
-    return $ret;
-}
-
-sub _format_multicheck_failure_summary {
-    my $self = shift;
-    my $format = shift;
-    my $nr_checks = shift;
-    my $error_stat = shift;
-
-    my $ret = $format;
-
-    my $details = "";
-    my $total_errors = 0;
-    for my $code (UNKNOWN,CRITICAL,WARNING) {
-        if (my $errs = $error_stat->{$code}) {
-            $details .= scalar(@$errs) . " " . $STATUS_TEXT{$code} . " (" . join (",",@$errs) . "), ";
-            $total_errors += scalar(@$errs);
+        my $nr_warnings = scalar(@{$np->messages->{warning} || []});
+        my $nr_errors = scalar(@{$np->messages->{critical} || []});
+        my @parts;
+        my $extra = "";
+        my $nr = 0;
+        for my $code (CRITICAL,WARNING,UNKNOWN) {
+            if (my $errs = $error_stat->{$code}) {
+                $extra .= scalar(@$errs) . " " . $STATUS_TEXT{$code} . " (" . join (",",@$errs) . "), ";
+                $nr += scalar(@$errs);
+            }
         }
+        if ($nr > 0) {
+            # Cut off extra chars at the end
+            $extra = substr($extra,0,-2);
+        }
+        $summary = $nr . " of " . $nr_checks . " failed: " . $extra;
     }
-    if ($total_errors > 0) {
-        # Cut off extra chars at the end
-        $details = substr($details,0,-2);
-    }
-    
-    $ret =~ s/\%d/$details/g;
-    $ret =~ s/\%e/$total_errors/g;
-    $ret =~ s/\%n/$nr_checks/g;
-    return $ret;
+    return ($code,$summary . "\n" . $message);    
 }
 
 # Create a formatted prefix for multicheck output
@@ -237,15 +184,8 @@ sub _multi_check_prefix {
     my $check = shift;
     my $idx = shift;
     my $max = shift;
-    
-    my $c = $check->{config};
-
+    my $label = $check->{config}->{key} || $check->{config}->{name} || "";
     my $l = length($max);
-    
-    return sprintf("[%$l.${l}s] %%c ",$idx)
-      if (defined($c->{multicheckprefix}) && !length($c->{multicheckprefix}));
-    
-    my $label =  $c->{multicheckprefix} || $c->{name} || $c->{key} || "";
     return sprintf("[%$l.${l}s] %%c %s: ",$idx,$label);
 }
 
@@ -263,67 +203,15 @@ sub _send_requests {
         }
         $start_time = [gettimeofday];
     }
-    # Detangle request for direct method calls and JMX requests to call:
-    my $req_map = $self->_detangle_requests(\@requests);
-    my @responses = ();
-    
-    $self->_execute_scripts(\@responses,$req_map);
-    $self->_execute_requests(\@responses,$req_map,$jmx);
-
+    my @responses = $jmx->request(@requests);
     if ($o->verbose) {
         print "Result fetched in ",tv_interval($start_time) * 1000," ms:\n";
         print Dumper(\@responses);
     }
+    #print Dumper(\@responses);
     return \@responses;
 }
 
-# Split up request for code-requests (i.e. scripts given in the configuration)
-# and 'real' requests. Remember the index, too so that the response can be
-# weave together
-sub _detangle_requests {
-    my $self = shift;
-    my $requests = shift;
-    my $req_map = {};
-    my $idx = 0;
-    for my $r (@$requests) {
-        push @{$req_map->{ref($r) eq "CODE" ? "code" : "request"}},[$r,$idx];
-        $idx++;
-    }
-    return $req_map;
-}
-
-# Execute subrefs created out of scripts. Put it in the right place of the
-# result array according to the remembered index
-sub _execute_scripts {
-    my $self = shift;
-    my $responses = shift;
-    my $req_map = shift;
-    for my $e (@{$req_map->{"code"}}) {
-        # Will die on error which will bubble up
-        $responses->[$e->[1]] = &{$e->[0]}();;
-    }    
-}
-
-# Execute requests and put it in the received responses in the right place for
-# the returned array. The index has been extracted beforehand and stored in the 
-# given req_map
-sub _execute_requests {
-    my $self = shift;
-    my $responses = shift;
-    my $req_map = shift;
-    my $jmx = shift;
-
-    # Call remote JMX and weave in
-    my $reqs2send = $req_map->{"request"};
-    if ($reqs2send) {
-        my @resp_received = $jmx->request(map { $_->[0] } @$reqs2send);
-        for my $r (@$reqs2send) {
-            $responses->[$r->[1]] = shift @resp_received;
-        }
-    }    
-}
-
-
 # Print online manual and exit (somewhat crude, I know)
 sub _print_doc_and_exit {
     my $self = shift;
@@ -365,11 +253,10 @@ sub _verify_and_initialize {
 
     # Fetch configuration
     my $config = $self->_get_config($o->config);
+    
     # Now, if a specific check is given, extract it, too.
     my $check_configs;
-    #print Dumper($config);
     $check_configs = $self->_extract_checks($config,$o->check);
-    #print Dumper($check_configs);
     if ($check_configs) {
         for my $c (@$check_configs) {
             my $s_c = new JMX::Jmx4Perl::Nagios::SingleCheck($np,$c);
@@ -390,12 +277,22 @@ sub _verify_and_initialize {
     for my $check (@{$self->{checks}}) {
         my $name = $check->name ? " [Check: " . $check->name . "]" : "";
         $self->nagios_die("An MBean name and a attribute/operation must be provided " . $name)
-          if ((!$check->mbean || (!$check->attribute && !$check->operation)) && !$check->alias && !$check->value && !$check->script);
+          if ((!$check->mbean || (!$check->attribute && !$check->operation)) && !$check->alias && !$check->value);
+        $self->verify_check($check,$name);
     }
 }
 
+sub verify_check {
+    my $self = shift;
+    my $check = shift;
+    my $name = shift;
+    my $np = $self->{np};
+    $self->nagios_die("At least a critical or warning threshold must be given " . $name) 
+      if ((!defined($check->critical) && !defined($check->warning)));        
+}
+
 # Extract one or more check configurations which can be 
-# simple <Check>s or <MultiCheck>s
+# simle <Check>s or <MultiCheck>s
 sub _extract_checks {
     my $self = shift;
     my $config = shift;
@@ -405,11 +302,10 @@ sub _extract_checks {
     if ($check) {
         $self->nagios_die("No configuration given") unless $config;
         $self->nagios_die("No checks defined in configuration") unless $config->{check};
-
+        
         my $check_configs;
         unless ($config->{check}->{$check}) {
             $check_configs = $self->_resolve_multicheck($config,$check,$self->{cmd_args});
-            $self->_retrieve_mc_summary_label($config,$check);
         } else {
             my $check_config = $config->{check}->{$check};
             $check_configs = ref($check_config) eq "ARRAY" ? $check_config : [ $check_config ];
@@ -417,8 +313,6 @@ sub _extract_checks {
         }
         $self->nagios_die("No check configuration with name " . $check . " found") unless (@{$check_configs});
 
-        #print Dumper($check_configs);
-
         # Resolve parent values
         for my $c (@{$check_configs}) {
             #print "[A] ",Dumper($c);
@@ -429,7 +323,7 @@ sub _extract_checks {
             for my $k (keys(%$c)) {
                 $c->{$k} = $self->_replace_placeholder($c->{$k},undef) unless ref($c->{$k});
             }
-            #print "[C] ",Dumper($c);
+            # print "[C] ",Dumper($c);
         }
         return $check_configs;
     } else {
@@ -449,28 +343,30 @@ sub _resolve_multicheck {
     if ($multi_checks)  {
         my $m_check = $multi_checks->{$check};
         if ($m_check) {
-            # Resolve all checks
-            my $c_names = [];
-            for my $type( qw(check multicheck)) {
-                if ($m_check->{$type}) {
-                    push @$c_names, ref($m_check->{$type}) eq "ARRAY" ? @{$m_check->{$type}} : $m_check->{$type};
-                }
-            }
-            for my $name (@$c_names) {
-                my ($c_name,$c_args) = $self->_parse_check_ref($name);
-                my $args_merged = $self->_merge_multicheck_args($c_args,$args);
-                $self->nagios_die("Unknown check '" . $c_name . "' for multi check " . $check) 
-                  unless defined($config->{check}->{$c_name}) or defined($multi_checks->{$c_name});
-                if ($config->{check}->{$c_name}) {
+            if ($m_check->{check}) {
+                # Resolve all checks
+                my $c_names = ref($m_check->{check}) eq "ARRAY" ? $m_check->{check} : [ $m_check->{check} ];
+                for my $name (@$c_names) {
+                    my ($c_name,$c_args) = $self->_parse_check_ref($name);
+                    my $args_merged = $self->_merge_multicheck_args($c_args,$args);
                     # We need a copy of the check hash to avoid mangling it up
                     # if it is referenced multiple times
+                    $self->nagios_die("Unknown check '" . $c_name . "' for multi check " . $check) 
+                      unless defined($config->{check}->{$c_name});
                     my $check = { %{$config->{check}->{$c_name}} };
                     $check->{key} = $c_name;
                     $check->{args} = $args_merged;
                     push @{$check_config},$check;
-                } else {
-                    # It's a multi check referenced via <Check> or <MultiCheck> ....
-                    push @{$check_config},@{$self->_resolve_multicheck($config,$c_name,$args_merged)};
+                }
+            }
+            if ($m_check->{multicheck}) {
+                my $mc_names = ref($m_check->{multicheck}) eq "ARRAY" ? $m_check->{multicheck} : [ $m_check->{multicheck} ];
+                for my $name (@$mc_names) {                    
+                    my ($mc_name,$mc_args) = $self->_parse_check_ref($name);
+                    my $args_merged = $self->_merge_multicheck_args($mc_args,$args);
+                    $self->nagios_die("Unknown multi check '" . $mc_name . "'")
+                      unless $multi_checks->{$mc_name};
+                    push @{$check_config},@{$self->_resolve_multicheck($config,$mc_name,$args_merged)};
                 }
             }
         }
@@ -478,24 +374,6 @@ sub _resolve_multicheck {
     return $check_config;
 }
 
-sub _retrieve_mc_summary_label { 
-    my $self = shift;
-    my $config = shift;
-    my $check = shift;
-
-    my $multi_checks = $config->{multicheck};    
-    if ($multi_checks) { 
-        my $m_check = $multi_checks->{$check};
-        if ($m_check && ($m_check->{summaryok} || $m_check->{summaryfailure})) {
-            my $mc_labels = 
-            $self->{multi_check_labels} = {
-                                           summary_ok => $m_check->{summaryok},
-                                           summary_failure => $m_check->{summaryfailure}
-                                          };
-        }
-    }
-}
-
 sub _merge_multicheck_args {
     my $self = shift;
     my $check_params = shift;
@@ -548,7 +426,7 @@ sub _resolve_check_config {
         }
         # Replace inherited values
         for my $k (keys %$parent_merged) {
-            my $parent_val = defined($parent_merged->{$k}) ?  $parent_merged->{$k} :  "";
+            my $parent_val = $parent_merged->{$k} || "";
             if (defined($check->{$k})) {
                 $check->{$k} =~ s/\$BASE/$parent_val/g;
             } else {
@@ -579,37 +457,21 @@ sub _replace_placeholder {
     my $self = shift;
     my $val = shift;
     my $args = shift;
+
     my $index = defined($args) ? join "|",0 ... $#$args : "\\d+";
 
-    my $regexp_s = <<'EOP';
-^(.*?)                                 # Start containing no args
-
-\$(                                    # Variable starts with '$'
-   ($index)  |                         # $0         without default value
-   \{\s*($index)\s*                    # ${0:12300} with default value
-     (?:  :([^\}]+)  )*\}              # ?: --> clustering group, optional (${0} is also ok)
-  )
-
-(.*|$)                                 # The rest which will get parsed next
-EOP
-    $regexp_s =~ s/\$index/$index/g;
-    my $regexp = qr/$regexp_s/sx;
+    my $regexp;
+    eval '$regexp = qr/^(.*?)\$((' . $index . ')|\{\s*(' . $index . ')\s*:([^\}]+)\})(.*|$)/';
     die "Cannot create placeholder regexp" if $@;
     my $rest = $val;
     my $ret = "";
-    while (defined($rest) && length($rest) && $rest =~ $regexp) {  
-        # $1: start with no placeholder
-        # $2: literal variable as it is defined
-        # $3: variable name (0,1,2,3,...)
-        # $4: same as $3, but either $3 or $4 is defined
-        # $5: default value (if any)
-        # $6: rest which is processed next in the loop
+    while (defined($rest) && length($rest) && $rest =~ /$regexp/) {        
         my $start = defined($1) ? $1 : "";
         my $orig_val = '$' . $2;
         my $i = defined($3) ? $3 : $4;
         my $default = $5;
-        my $end = defined($6) ? $6 : "";
         $default =~ s/^\s*(.*)+?\s*$/$1/ if $default; # Trim whitespace
+        my $end = defined($6) ? $6 : "";
         #print Dumper({start => $start, orig => $orig_val,end => $end, default=> $default, rest => $rest, i => $i}); 
         if (defined($args)) {
             my $repl = $args->[$i];            
@@ -639,7 +501,7 @@ EOP
             } else {
                 if (!length($ret)) {
                     # No default value, nothing else for this value. We
-                    # consider it undefined
+                    # consider at undefined
                     return undef;
                 }
             }
@@ -654,7 +516,7 @@ EOP
 sub _parse_check_ref {
     my $self = shift;
     my $check_ref = shift;
-    if ($check_ref =~/^\s*(.+?)\((.*)\)\s*$/) {
+    if ($check_ref =~/^\s*([^(]+)\(([^)]*)\)\s*$/) {
         my $name = $1;
         my $args_s = $2;
         my $args = [ parse_line('\s*,\s*',0,$args_s) ];
@@ -679,7 +541,7 @@ sub _server_config {
     return shift->{server_config};
 }
 
-# Create the nagios plugin used for preparing the nagios output
+# Create the nagios plugin used for preparing the nagious output
 sub create_nagios_plugin {
     my $self = shift;
     my $np = Nagios::Plugin->
@@ -704,7 +566,6 @@ sub create_nagios_plugin {
     $np->shortname(undef);
     $self->add_common_np_args($np);
     $self->add_nagios_np_args($np);
-    $np->{msg_handler} = new JMX::Jmx4Perl::Nagios::MessageHandler();
     $np->getopts();
     return $np;
 }
@@ -810,20 +671,8 @@ sub add_nagios_np_args {
 
     $np->add_arg(
                  spec => "base|base-alias|b=s",
-                 help => "Base name, which when given, interprets critical and warning values as relative in the range 0 .. 100%. Must be given in the form mbean/attribute/path",
+                 help => "Base alias name, which when given, interprets critical and warning values as relative in the range 0 .. 100%",
                 );
-    $np->add_arg(
-                 spec => "base-mbean=s",
-                 help => "Base MBean name, interprets critical and warning values as relative in the range 0 .. 100%. Requires a base-attribute, too",
-                );
-    $np->add_arg(
-                 spec => "base-attribute=s",
-                 help => "Base attribute for a relative check. Used together with base-mbean",
-                );
-    $np->add_arg(
-                 spec => "base-path=s",
-                 help => "Base path for relatie checks, where this path is used on the base attribute's value",
-                );    
     $np->add_arg(
                  spec => "unit=s",
                  help => "Unit of measurement of the data retreived. Recognized values are [B|KB|MN|GB|TB] for memory values and [us|ms|s|m|h|d] for time values"
@@ -854,10 +703,6 @@ sub add_nagios_np_args {
                  spec => "label|l=s",
                  help => "Label to be used for printing out the result of the check. Placeholders can be used."
                 );
-    $np->add_arg(
-                 spec => "perfdata=s",
-                 help => "Whether performance data should be omitted, which are included by default."
-                );
     $np->add_arg(
                  spec => "unknown-is-critical",
                  help => "Map UNKNOWN errors to errors with a CRITICAL status"
@@ -920,7 +765,7 @@ sub AUTOLOAD {
     my $opts_name = $name;
     $opts_name =~ s/_/-/;
 
-    if ($SERVER_CONFIG_KEYS->{$name}) {
+    if ($SERVER_CONFIG_KEYS->{$name}) {        
         return $np->opts->{$opts_name} if $np->opts->{$opts_name};
         my $c = $SERVER_CONFIG_KEYS->{$name};
         if ($c) {
@@ -944,7 +789,7 @@ sub AUTOLOAD {
 sub nagios_die {
     my $self = shift;
     my @args = @_;
-
+    
     my $np = $self->{np};
     $np->nagios_die(join("",@args),$np->opts->{'unknown-is-critical'} ? CRITICAL : UNKNOWN)
 }
